@@ -1,6 +1,23 @@
+# This file is part of scrilla: https://github.com/chinchalinchin/scrilla.
+
+# scrilla is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 3
+# as published by the Free Software Foundation.
+
+# scrilla is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with scrilla.  If not, see <https://www.gnu.org/licenses/>
+# or <https://github.com/chinchalinchin/scrilla/blob/develop/main/LICENSE>.
+"""
+A module of functions that calculate financial statistics
+"""
 from scrilla import settings, services, files, cache, static, errors
-from scrilla.objects.cashflow import Cashflow
-import scrilla.analysis.statistics as statistics
+from scrilla.analysis.objects.cashflow import Cashflow
+import scrilla.analysis.models.geometric.statistics as statistics
 import scrilla.util.outputter as outputter
 
 MODEL_DDM="ddm"
@@ -12,7 +29,7 @@ profile_cache = cache.ProfileCache()
 
 # NOTE: if ticker_profile is provided, it effectively nullifies start_date and end_date.
 # TODO: pass in risk_free_rate=None as optional argument to prevent overusing services
-def sharpe_ratio(ticker, start_date=None, end_date=None, risk_free_rate=None, ticker_profile=None):
+def sharpe_ratio(ticker, start_date=None, end_date=None, risk_free_rate=None, ticker_profile=None, method=settings.ESTIMATION_METHOD):
     """
     Description
     -----------
@@ -42,14 +59,13 @@ def sharpe_ratio(ticker, start_date=None, end_date=None, risk_free_rate=None, ti
     except errors.InputValidationError as ive:
         raise ive
 
-    result = profile_cache.filter_profile_cache(ticker=ticker, start_date=start_date, end_date=end_date)
+    result = profile_cache.filter_profile_cache(ticker=ticker, start_date=start_date, end_date=end_date, method=method)
 
     if result is not None and result[static.keys['STATISTICS']['SHARPE']] is not None:
         return result[static.keys['STATISTICS']['SHARPE']]
 
     if ticker_profile is None:  
-        ticker_profile = statistics.calculate_risk_return(ticker=ticker, start_date=start_date,
-                                                        end_date=end_date)
+        ticker_profile = statistics.calculate_risk_return(ticker=ticker, start_date=start_date, end_date=end_date, method=method)
 
     if risk_free_rate is None:
         risk_free_rate = services.get_risk_free_rate()
@@ -57,12 +73,12 @@ def sharpe_ratio(ticker, start_date=None, end_date=None, risk_free_rate=None, ti
     sharpe_ratio = (ticker_profile['annual_return'] - risk_free_rate)/ticker_profile['annual_volatility']
 
     profile_cache.save_or_update_row(ticker=ticker, start_date=start_date,
-                                        end_date=end_date,sharpe_ratio=sharpe_ratio)
+                                        end_date=end_date,sharpe_ratio=sharpe_ratio, method=method)
 
     return sharpe_ratio
 
 # if no dates are specified, defaults to last 100 days
-def market_premium(start_date=None, end_date=None, market_profile = None):
+def market_premium(start_date=None, end_date=None, market_profile = None, method=settings.ESTIMATION_METHOD):
     """
     Description
     -----------
@@ -91,8 +107,7 @@ def market_premium(start_date=None, end_date=None, market_profile = None):
 
     if market_profile is None:
         try:
-            market_profile = statistics.calculate_risk_return(ticker=settings.MARKET_PROXY, 
-                                                                start_date=start_date, end_date=end_date)
+            market_profile = statistics.calculate_risk_return(ticker=settings.MARKET_PROXY, start_date=start_date, end_date=end_date, method=method)
         except errors.SampleSizeError as se:
             raise se
         except errors.PriceError as pe:
@@ -104,7 +119,7 @@ def market_premium(start_date=None, end_date=None, market_profile = None):
     return market_premium
         
 
-def market_beta(ticker, start_date=None, end_date=None, market_profile=None, market_correlation=None, ticker_profile=None, sample_prices=None):
+def market_beta(ticker, start_date=None, end_date=None, market_profile=None, market_correlation=None, ticker_profile=None, sample_prices=None, method=settings.ESTIMATION_METHOD):
     """
     Description
     -----------
@@ -134,7 +149,7 @@ def market_beta(ticker, start_date=None, end_date=None, market_profile=None, mar
     except errors.InputValidationError as ive:
         raise ive
 
-    result = profile_cache.filter_profile_cache(ticker=ticker, start_date=start_date, end_date=end_date)
+    result = profile_cache.filter_profile_cache(ticker=ticker, start_date=start_date, end_date=end_date, method=method)
 
     if result is not None and result[static.keys['STATISTICS']['BETA']] is not None:
         return result[static.keys['STATISTICS']['BETA']]
@@ -142,20 +157,18 @@ def market_beta(ticker, start_date=None, end_date=None, market_profile=None, mar
     try:
         if market_profile is None:
             if sample_prices is None:
-                market_profile = statistics.calculate_risk_return(ticker=settings.MARKET_PROXY, 
-                                                                start_date=start_date, 
-                                                                end_date=end_date)
+                market_profile = statistics.calculate_risk_return(ticker=settings.MARKET_PROXY, start_date=start_date, 
+                                                        end_date=end_date, method=method)
             else:   
-                market_profile = statistics.calculate_risk_return(ticker=settings.MARKET_PROXY,
+                market_profile = statistics.calculate_risk_return(ticker=settings.MARKET_PROXY, method=method,
                                                                 sample_prices=sample_prices[settings.MARKET_PROXY])
         if ticker_profile is None:
             if sample_prices is None:
-                ticker_profile = statistics.calculate_risk_return(ticker=ticker,
-                                                                start_date=start_date,
-                                                                end_date=end_date)
+                ticker_profile = statistics.calculate_risk_return(ticker=ticker,start_date=start_date,
+                                                        end_date=end_date, method=method)
             else:
-                ticker_profile = statistics.calculate_risk_return(ticker=ticker, 
-                                                                sample_prices=sample_prices[ticker])
+                ticker_profile = statistics.calculate_risk_return(ticker=ticker, method=method,
+                                                        sample_prices=sample_prices[ticker])
 
         market_covariance = statistics.calculate_return_covariance(ticker_1=ticker, ticker_2=settings.MARKET_PROXY,
                                                                     profile_1=ticker_profile, profile_2=market_profile,
@@ -171,11 +184,11 @@ def market_beta(ticker, start_date=None, end_date=None, market_profile=None, mar
 
     beta = market_covariance / (market_profile['annual_volatility']**2)
 
-    profile_cache.save_or_update_row(ticker=ticker, start_date=start_date, end_date=end_date, asset_beta=beta)
+    profile_cache.save_or_update_row(ticker=ticker, start_date=start_date, end_date=end_date, asset_beta=beta, method=method)
 
     return beta
 
-def cost_of_equity(ticker, start_date=None, end_date=None, market_profile=None, market_correlation=None):
+def cost_of_equity(ticker, start_date=None, end_date=None, market_profile=None, market_correlation=None, method=settings.ESTIMATION_METHOD):
     """
     Description
     -----------
@@ -198,15 +211,16 @@ def cost_of_equity(ticker, start_date=None, end_date=None, market_profile=None,
     except errors.InputValidationError as ive:
         raise ive
 
-    result = profile_cache.filter_profile_cache(ticker=ticker, start_date=start_date, end_date=end_date)
+    result = profile_cache.filter_profile_cache(ticker=ticker, start_date=start_date, end_date=end_date, method=method)
 
     if result is not None and result[static.keys['STATISTICS']['EQUITY']] is not None:
         return result[static.keys['STATISTICS']['EQUITY']]
 
     try:
         beta = market_beta(ticker=ticker, start_date=start_date, end_date=end_date,
-                            market_profile=market_profile, market_correlation=market_correlation)
-        premium = market_premium(start_date=start_date, end_date=end_date, market_profile=market_profile)
+                            market_profile=market_profile, market_correlation=market_correlation,
+                            method=method)
+        premium = market_premium(start_date=start_date, end_date=end_date, market_profile=market_profile, method=method)
     except errors.SampleSizeError as se:
         raise se
     except errors.PriceError as pe:
@@ -216,7 +230,7 @@ def cost_of_equity(ticker, start_date=None, end_date=None, market_profile=None,
 
     equity_cost = (premium*beta + services.get_risk_free_rate())
 
-    profile_cache.save_or_update_row(ticker=ticker, start_date=start_date, end_date=end_date, equity_cost=equity_cost)
+    profile_cache.save_or_update_row(ticker=ticker, start_date=start_date, end_date=end_date, equity_cost=equity_cost, method=method)
     return equity_cost
 
 def screen_for_discount(model=None, discount_rate=None):
