@@ -1,17 +1,113 @@
-import scipy.optimize as optimize
-from scrilla import static
 
-import settings
-import util.outputter as outputter
+# This file is part of scrilla: https://github.com/chinchalinchin/scrilla.
+
+# scrilla is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 3
+# as published by the Free Software Foundation.
+
+# scrilla is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with scrilla.  If not, see <https://www.gnu.org/licenses/>
+# or <https://github.com/chinchalinchin/scrilla/blob/develop/main/LICENSE>.
+
+"""
+A module of functions that wrap around `scipy.optimize` in order to optimize statistical and financial functions of interest.
+"""
+
+import scipy.optimize as optimize
+from numpy import sqrt
+from scrilla import static, settings
+from scrilla.analysis import estimators
+import scrilla.util.outputter as outputter
 
 logger = outputter.Logger('optimizer', settings.LOG_LEVEL)
 
+def maximize_univariate_normal_likelihood(data):
+    """
+    Maximizes the normal (log-)likelihood of the sample with respect to the mean and volatility in order to estimate the mean and volatility of the population distribution described by the sample.
+
+    Parameters
+    ----------
+    1. **data**: ``list``
+        Sample of data drawn from a univariate normal population. 
+
+    Returns
+    -------
+    ``list`` : A list containing the maximum likelihood estimates of the normal distribution's parameters. The first element of the list corresponds to the mean and the second element corresponds to the volatility.
+
+    .. notes::
+        * Some comments about the methodology. This module assumes an underlying asset price process that follows Geometric Brownian motion. This implies the return on the asset over intervals of  \\(\delta t\\) is normally distributed with mean \\(\mu * \delta t\\) and volatility \\(\sigma \cdot \sqrt{\delta t}\\). If the sample is scaled by \\(\delta t\\), then the mean becomes \\(\mu\\) and the volatility \\(\frac{\sigma}{\sqrt t}\\).  Moreover, increments are independent. Therefore, if the observations are made over equally spaced intervals, each observation is drawn from an independent, identially distributed normal random variable. The parameters \\(\mu\\) and :\\(\frac {\sigma}{\delta t}\\) can then be estimated by maximizing the probability of observing a given sample with respect to the parameters. To obtain the estimate for \\(\sigma\\), multiply the result of this function by \\(\delta t\\).
+        * Theoretically, the output of this function should equal the same value obtained from the method of moment matching. However, there is a small discrepancy. It could be due to floating point arthimetic. However, see Section 2.2 of the following for what I think may be, if not the source, at least related to the [issue](https://www.researchgate.net/publication/5071468_Maximum_Likelihood_Estimation_of_Generalized_Ito_Processes_With_Discretely-Sampled_Data)
+        The author, however, is not considering the transformed Ito process, the log of the asset price process. It seems like his conclusion may be an artifact of Ito's Lemma? Not sure. Will need to think.
+    """
+
+    likelihood = lambda x: (-1)*estimators.univariate_normal_likelihood_function(params=x, data=data)
+
+    # make an educated guess
+    first_quartile = estimators.sample_percentile(data=data, percentile=0.25)
+    median = estimators.sample_percentile(data=data, percentile=0.5)
+    third_quartile = estimators.sample_percentile(data, percentile=0.75)
+    guess = [median, (third_quartile-first_quartile)/2]
+
+    params = optimize.minimize(fun = likelihood, x0 = guess, options={'disp': False},
+                                    method=static.constants['OPTIMIZATION_METHOD'])
+    return params.x
+
+def maximize_bivariate_normal_likelihood(data):
+    """
+
+    .. warning ::
+        This does not work. Son of a gun.
+
+    """
+
+    x_data = [ datum[0] for datum in data ]
+    y_data = [ datum[1] for datum in data ] 
+
+    x_25_percentile = estimators.sample_percentile(x_data, 0.25)
+    y_25_percentile = estimators.sample_percentile(y_data, 0.25)
+    x_median = estimators.sample_percentile(x_data, 0.5)
+    y_median = estimators.sample_percentile(y_data, 0.5)
+    x_75_percentile = estimators.sample_percentile(x_data, 0.75)
+    y_75_percentile = estimators.sample_percentile(y_data, 0.75)
+    x_1_percentile = estimators.sample_percentile(x_data, 0.01)
+    y_1_percentile = estimators.sample_percentile(y_data, 0.01)
+    x_99_percentile = estimators.sample_percentile(x_data, 0.99)
+    y_99_percentile = estimators.sample_percentile(y_data, 0.99)
+
+    likelihood = lambda x : (-1)*estimators.bivariate_normal_likelihood_function(params=x, 
+                                                                                data=data)
+
+    var_x_guess = (x_75_percentile - x_25_percentile)/2
+    var_y_guess = (y_75_percentile - y_25_percentile)/2
+    guess = [ x_median, y_median, var_x_guess, var_y_guess, 0]
+    var_x_bounds = x_99_percentile - x_1_percentile
+    var_y_bounds = y_99_percentile - y_1_percentile
+    cov_bounds = sqrt(var_x_bounds*var_y_bounds)
+
+    params = optimize.minimize(fun = likelihood, 
+                                x0 = guess,
+                                bounds = [
+                                    (None, None),
+                                    (None, None),
+                                    (0, var_x_bounds),
+                                    (0, var_y_bounds),
+                                    (-cov_bounds, cov_bounds)
+                                ],
+                                options ={'disp': False},
+                                method='Nelder-Mead')
+    return params.x
+    
 def optimize_portfolio_variance(portfolio, target_return=None):
     """
     Parameters
     ----------
     1. portfolio : Portfolio \n
-        An instance of the Portfolio class defined in  objects.portfolio. Must be initialized with an array of ticker symbols. Optionally, it can be initialized with a start_date and end_date datetime. If start_date and end_date are specified, the portfolio will be optimized over the stated time period.\n \n
+        An instance of the Portfolio class defined in  analysis.objects.portfolio. Must be initialized with an array of ticker symbols. Optionally, it can be initialized with a start_date and end_date datetime. If start_date and end_date are specified, the portfolio will be optimized over the stated time period.\n \n
     2. target_return : float \n
         The target return, as a decimal, subject to which the portfolio's volatility will be minimized.
 
@@ -52,7 +148,7 @@ def optimize_conditional_value_at_risk(portfolio, prob, expiry, target_return=No
     Parameters
     ----------
     1, portfolio : Portfolio \n
-        An instance of the Portfolio class defined in  objects.portfolio. Must be initialized with an array of ticker symbols. Optionally, it can be initialized with a start_date and end_date datetime. If start_date and end_date are specified, the portfolio will be optimized over the stated time period.\n \n
+        An instance of the Portfolio class defined in  analysis.objects.portfolio. Must be initialized with an array of ticker symbols. Optionally, it can be initialized with a start_date and end_date datetime. If start_date and end_date are specified, the portfolio will be optimized over the stated time period.\n \n
     2. prob: float \n
         Confidence level for value at risk. \n \n
     3. expiry: float \n
@@ -91,7 +187,7 @@ def maximize_sharpe_ratio(portfolio, target_return=None):
     Parameters
     ----------
     1. portfolio : Portfolio \n
-        An instance of the Portfolio class defined in  objects.portfolio. Must be initialized with an array of ticker symbols. Optionally, it can be initialized with a start_date and end_date datetime. If start_date and end_date are specified, the portfolio will be optimized over the stated time period.\n \n
+        An instance of the Portfolio class defined in  analysis.objects.portfolio. Must be initialized with an array of ticker symbols. Optionally, it can be initialized with a start_date and end_date datetime. If start_date and end_date are specified, the portfolio will be optimized over the stated time period.\n \n
     2. target_return : float \n
         The target return, as a decimal, subject to which the portfolio's volatility will be minimized.
 
@@ -133,7 +229,7 @@ def maximize_portfolio_return(portfolio):
     Parameters
     ----------
     * portfolio : Portfolio \n
-        An instance of the Portfolio class defined in  objects.portfolio. Must be initialized with an array of ticker symbols. Optionally, it can be initialized with a start_date and end_date datetime. If start_date and end_date are specified, the portfolio will be optimized over the stated time period.\n \n
+        An instance of the Portfolio class defined in  analysis.objects.portfolio. Must be initialized with an array of ticker symbols. Optionally, it can be initialized with a start_date and end_date datetime. If start_date and end_date are specified, the portfolio will be optimized over the stated time period.\n \n
 
     Output
     ------
@@ -160,7 +256,7 @@ def calculate_efficient_frontier(portfolio, steps=None):
     Parameters
     ----------
     1. portfolio : Portfolio \n
-        An instance of the Portfolio class defined in  objects.portfolio. Must be initialized with an array of ticker symbols. Optionally, it can be initialized with a start_date and end_date datetime. If start_date and end_date are specified, the portfolio will be optimized over the stated time period.\n \n
+        An instance of the Portfolio class defined in  analysis.objects.portfolio. Must be initialized with an array of ticker symbols. Optionally, it can be initialized with a start_date and end_date datetime. If start_date and end_date are specified, the portfolio will be optimized over the stated time period.\n \n
 
     2. steps : int \n
         The number of points calculated in the efficient frontier. If none is provided, it defaults to the environment variable FRONTIER_STEPS.
