@@ -1,43 +1,58 @@
+# This file is part of scrilla: https://github.com/chinchalinchin/scrilla.
+
+# scrilla is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 3
+# as published by the Free Software Foundation.
+
+# scrilla is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with scrilla.  If not, see <https://www.gnu.org/licenses/>
+# or <https://github.com/chinchalinchin/scrilla/blob/develop/main/LICENSE>.
+
+"""
+# CLI Entrypoint 
+This script acts as the entrypoint for the CLI application and contains the majority of the control structures for the program. It parses the arguments supplied through the command line, delegates them to the appropriate application function and then passes the results to the `scrilla.outputter` module functions for formatting and printing to screen.
+
+The arguments are parsed in such a way that arguments which are not supplied are set to None. All application functions are set up to accept None as a value for their optional arguments. This makes passing arguments to application functions easier as the `main.py` script doesn't have to worry about their values. In other words, `main.py` always passes all arguments to application functions, even if they aren't supplied through the command line; it just sets the ones which aren't supplied to None.
+"""
+
 import sys, os, traceback, json
+from typing import Callable
 
-#  Note: need to import from package when running from wheel.
-# if running locally through main.py file, these imports should be replaced
-#       from . import settings, from . import services, from . import files
-# annoying, but it is what it is.
-if __name__=="__main__":
-    import settings, services, files, static
-    from errors import APIResponseError, ConfigurationError, InputValidationError, SampleSizeError, PriceError
+from scrilla import settings, services, files, static
+from scrilla.errors import APIResponseError, ConfigurationError, InputValidationError, SampleSizeError, PriceError
+from scrilla.util import helper, outputter, formatter
+from scrilla.analysis import optimizer, markets, estimators
+from scrilla.analysis.models.geometric import statistics, probability
+# TODO: conditional imports based on value of ANALYSIS_MODE
 
-else:
-    from scrilla import settings, services, files, static
-    from scrilla.errors import APIResponseError, ConfigurationError, InputValidationError, SampleSizeError, PriceError
-
-
-from util import helper, outputter, formatter
-from analysis import statistics, optimizer, markets, blackscholes
-from objects.portfolio import Portfolio
-from objects.cashflow import Cashflow
+from scrilla.analysis.objects.portfolio import Portfolio
+from scrilla.analysis.objects.cashflow import Cashflow
 
 if settings.APP_ENV != "container":
     import util.plotter as plotter
 
 logger = outputter.Logger('main', settings.LOG_LEVEL)
 
-"""
-main.py
--------
-CLI Entrypoint 
---------------
-Description
------------
-This script acts as the entrypoint for the CLI application. It parses the arguments supplied through the command line, delegates them to the appropriate application function and then passes the results to the Logger class for formatting and printing to screen. \n \n 
-
-The arguments are parsed in such a way that arguments which are not supplied are set to None. All application functions are set up to accept None as a value for their optional arguments. This makes passing arguments to application functions easier as the `main.py` script doesn't have to worry about their values. In other words, `main.py` always passes all arguments to application functions, even if they aren't supplied through the command line; it just sets the ones which aren't supplied to None.  \n \n
-"""
 non_container_functions = [formatter.FUNC_ARG_DICT['plot_dividends'], formatter.FUNC_ARG_DICT['plot_moving_averages'],
                                formatter.FUNC_ARG_DICT['plot_risk_profile'], formatter.FUNC_ARG_DICT['plot_frontier']]
 
-def validate_function_usage(selection, args, wrapper_function, required_length=1, exact=False):
+def validate_function_usage(selection: str, args: list, wrapper_function: Callable[[], None], required_length: int=1, exact: bool=False) -> None:
+    """
+    Parameters
+    ----------
+    1. **selection** : ``str``
+    2. **args**: ``list``
+        List of ticker/statistic symbols whose length is be to validated.
+    3. **wrapper_function** : ``func()``
+    4. **required_length** : ``int``
+    5. **exact** : ``bool``
+        *Optional*. If the required length constraint is an equality, set to `True`. If the constraint is an inequality, set to `False`. Defaults to `False`. 
+    """
     if selection in non_container_functions and settings.APP_ENV == 'container':
         logger.comment('Graphics functionality disabled when application is containerized.')
 
@@ -55,13 +70,40 @@ def validate_function_usage(selection, args, wrapper_function, required_length=1
             print(str(e))
             traceback.print_exc()
 
-def print_format_to_screen(xtra_dict):
+def print_format_to_screen(xtra_dict: dict):
+    """
+    Checks if the inputted optional arguments allow printing pretty formatted text to screen.
+
+    Parameters
+    ----------
+    1. **xtra_dict** : ``dict``
+        Formatted dictionary containing optional arguments. Result of a call to `helper.format_xtra_args_dict`.
+    
+    Returns
+    -------
+    ``bool``
+    """
     return xtra_dict['json'] is None and xtra_dict['suppress'] is None
 
-def print_json_to_screen(xtra_dict):
+def print_json_to_screen(xtra_dict: dict):
+    """
+    Checks if inputted optional arguments allow printing json formatted text to screen.
+    
+    Parameters
+    ----------
+    1. **xtra_dict** : ``dict``
+        Formatted dictionary containing optional arguments. Result of a call to `helper.format_xtra_args_dict`.
+    
+    Returns
+    -------
+    ``bool``
+    """
     return xtra_dict['json'] is not None and xtra_dict['suppress'] is None
 
-def do_program():
+def do_program() -> None:
+    """
+    Parses command line arguments and passes the formatted arguments to appropriate function from the library.
+    """
     if len(sys.argv)>0:
         logger.debug('Parsing and invoking command line arguments')
         opt = sys.argv[1]
@@ -82,8 +124,8 @@ def do_program():
             logger.comment(f'Clearing {settings.STATIC_DIR}')
             files.clear_directory(directory=settings.STATIC_DIR, retain=True)
 
-        ### FUNCTION: Clear Watchlist
-        elif opt == formatter.FUNC_ARG_DICT["clear_watchlist"]:
+        ### FUNCTION: Clear Common
+        elif opt == formatter.FUNC_ARG_DICT["clear_common"]:
             logger.comment(f'Clearing {settings.COMMON_DIR}')
             files.clear_directory(directory=settings.COMMON_DIR, retain=True)
 
@@ -127,8 +169,8 @@ def do_program():
             
             args = sys.argv[2:]
             xtra_args, xtra_values, main_args = helper.separate_and_parse_args(args)
-            xtra_dict = helper.format_xtra_args_dict(xtra_args, xtra_values)
-            logger.log_arguments(main_args=main_args, xtra_args=xtra_args, xtra_values=xtra_values)
+            xtra_dict = helper.format_xtra_args_dict(xtra_args, xtra_values, settings.ESTIMATION_METHOD)
+            logger.log_arguments(main_args,xtra_args,xtra_values)
             exact, selected_function = False, None
 
             if print_format_to_screen(xtra_dict):
@@ -147,16 +189,21 @@ def do_program():
                 selected_function, required_length = cli_asset_type, 1
 
             ### FUNCTION: Black-Scholes Value At Risk
-            elif opt == formatter.FUNC_ARG_DICT['bs_var']:
-                def cli_bs_var():
+            elif opt == formatter.FUNC_ARG_DICT['var']:
+                def cli_var():
                     all_vars = {}
                     for arg in main_args:
-                        prices = services.get_daily_price_history(ticker=arg, start_date=xtra_dict['start_date'],
+                        prices = services.get_daily_price_history(ticker=arg, 
+                                                                    start_date=xtra_dict['start_date'],
                                                                     end_date=xtra_dict['end_date'])
                         latest_price = prices[helper.get_first_json_key(prices)]
-                        profile = statistics.calculate_risk_return(ticker=arg, sample_prices=prices)
-                        valueatrisk = blackscholes.percentile(S0=latest_price, vol=profile['annual_volatility'],
-                                                                ret=profile['annual_return'], expiry=xtra_dict['expiry'],
+                        profile = statistics.calculate_risk_return(ticker=arg, 
+                                                                    sample_prices=prices, 
+                                                                    method=xtra_dict['estimation'])
+                        valueatrisk = probability.percentile(S0=latest_price, 
+                                                                vol=profile['annual_volatility'],
+                                                                ret=profile['annual_return'], 
+                                                                expiry=xtra_dict['expiry'],
                                                                 percentile=xtra_dict['probability'])
                         all_vars[arg]=valueatrisk
 
@@ -169,22 +216,29 @@ def do_program():
                     if xtra_dict['save_file'] is not None:
                         files.save_file(file_to_save=all_vars, file_name=xtra_dict['save_file'])
 
-                selected_function, required_length = cli_bs_var, 2
+                selected_function, required_length = cli_var, 2
                 
             ### FUNCTION: Black-Scholes Conditional Value At Risk
-            elif opt == formatter.FUNC_ARG_DICT['bs_cvar']:
-                def cli_bs_var():
+            elif opt == formatter.FUNC_ARG_DICT['cvar']:
+                def cli_var():
                     all_cvars = {}
                     for arg in main_args:
-                        prices = services.get_daily_price_history(ticker=arg, start_date=xtra_dict['start_date'],
+                        prices = services.get_daily_price_history(ticker=arg, 
+                                                                    start_date=xtra_dict['start_date'],
                                                                     end_date=xtra_dict['end_date'])
                         latest_price = prices[helper.get_first_json_key(prices)]
-                        profile = statistics.calculate_risk_return(ticker=arg, sample_prices=prices)
-                        valueatrisk = blackscholes.percentile(S0=latest_price, vol=profile['annual_volatility'],
-                                                                ret=profile['annual_return'], expiry=xtra_dict['expiry'],
+                        profile = statistics.calculate_moment_risk_return(ticker=arg, 
+                                                                            sample_prices=prices, 
+                                                                            method=xtra_dict['estimation'])
+                        valueatrisk = probability.percentile(S0=latest_price, 
+                                                                vol=profile['annual_volatility'],
+                                                                ret=profile['annual_return'], 
+                                                                expiry=xtra_dict['expiry'],
                                                                 percentile=xtra_dict['probability'])
-                        cvar = blackscholes.conditional_expected_value(S0=latest_price, vol=profile['annual_volatility'],
-                                                                        ret=profile['annual_return'], expiry=xtra_dict['expiry'],
+                        cvar = probability.conditional_expected_value(S0=latest_price, 
+                                                                        vol=profile['annual_volatility'],
+                                                                        ret=profile['annual_return'], 
+                                                                        expiry=xtra_dict['expiry'],
                                                                         conditional_value=valueatrisk)
                         all_cvars[arg]=cvar
 
@@ -197,7 +251,7 @@ def do_program():
                     if xtra_dict['save_file'] is not None:
                         files.save_file(file_to_save=all_cvars, file_name=xtra_dict['save_file'])
 
-                selected_function, required_length = cli_bs_var, 2
+                selected_function, required_length = cli_var, 2
 
             ### FUNCTION: Capital Asset Pricing Model Cost of Equity
             elif opt == formatter.FUNC_ARG_DICT['capm_equity_cost']:
@@ -205,7 +259,8 @@ def do_program():
                     all_costs = {}
                     for arg in main_args:
                         equity_cost = markets.cost_of_equity(ticker=arg, start_date=xtra_dict['start_date'], 
-                                                                end_date=xtra_dict['end_date'])
+                                                                end_date=xtra_dict['end_date'], 
+                                                                method=xtra_dict['estimation'])
                         all_costs[arg] = equity_cost
 
                         if print_format_to_screen(xtra_dict):
@@ -226,8 +281,10 @@ def do_program():
                 def cli_capm_beta():
                     all_betas = {}
                     for arg in main_args:
-                        beta = markets.market_beta(ticker=arg, start_date=xtra_dict['start_date'], 
-                                                    end_date=xtra_dict['end_date'])
+                        beta = markets.market_beta(ticker=arg, 
+                                                    start_date=xtra_dict['start_date'], 
+                                                    end_date=xtra_dict['end_date'], 
+                                                    method=xtra_dict['estimation'])
                         all_betas[arg] = beta
 
                         if print_format_to_screen(xtra_dict):
@@ -262,17 +319,26 @@ def do_program():
             ### FUNCTION: Correlation Matrix
             elif opt == formatter.FUNC_ARG_DICT["correlation"]:
                 def cli_correlation():
-                    result = statistics.get_ito_correlation_matrix_string(tickers=main_args, indent=formatter.INDENT, 
-                                                                        start_date=xtra_dict['start_date'], 
-                                                                        end_date=xtra_dict['end_date'])
-                    outputter.print_below_new_line(f'\n{result}')
+                    matrix = statistics.correlation_matrix(tickers=main_args,
+                                                            start_date=xtra_dict['start_date'], 
+                                                            end_date=xtra_dict['end_date'],
+                                                            method=xtra_dict['estimation'])
+                    if print_format_to_screen(xtra_dict):
+                        outputter.correlation_matrix(tickers=main_args, correlation_matrix=matrix)
+    
+                    elif print_json_to_screen(xtra_dict):
+                        print(json.dumps(matrix))
+
+                    if xtra_dict['save_file'] is not None:
+                        files.save_file(file_to_save=matrix, file_name=xtra_dict['save_file'])
+                        
                 selected_function, required_length = cli_correlation, 2
 
             ### FUNCTION: Correlation Time Series
             elif opt == formatter.FUNC_ARG_DICT['correlation_time_series']:
                 def cli_correlation_series():
                     ticker_1, ticker_2 = main_args[0], main_args[1]
-                    result = statistics.calculate_ito_correlation_series(ticker_1=ticker_1,ticker_2=ticker_2,
+                    result = statistics.calculate_moment_correlation_series(ticker_1=ticker_1,ticker_2=ticker_2,
                                                                             start_date=xtra_dict['start_date'],
                                                                             end_date=xtra_dict['end_date'])
                     for date in result:
@@ -288,7 +354,7 @@ def do_program():
                     for arg in main_args:
                         dividends = services.get_dividend_history(arg)
                         if xtra_dict['discount'] is None:
-                            discount = markets.cost_of_equity(ticker=arg)
+                            discount = markets.cost_of_equity(ticker=arg, method=xtra_dict['estimation'])
                         else:
                             discount = xtra_dict['discount']
                         model_results[f'{arg}_discount_dividend'] = Cashflow(sample=dividends, 
@@ -323,19 +389,26 @@ def do_program():
             ### FUNCTION: Efficient Frontier
             elif opt == formatter.FUNC_ARG_DICT['efficient_frontier']:
                 def cli_efficient_frontier():
-                    portfolio = Portfolio(tickers=main_args, start_date=xtra_dict['start_date'], 
-                                            end_date=xtra_dict['end_date'])
-                    frontier = optimizer.calculate_efficient_frontier(portfolio=portfolio)
+                    portfolio = Portfolio(tickers=main_args, 
+                                            start_date=xtra_dict['start_date'], 
+                                            end_date=xtra_dict['end_date'],
+                                            method=xtra_dict['estimation'])
+                    frontier = optimizer.calculate_efficient_frontier(portfolio=portfolio, 
+                                                                        steps=xtra_dict['steps'])
 
                     if xtra_dict['suppress'] is None:
                         if xtra_dict['json'] is None:
-                            outputter.efficient_frontier(portfolio=portfolio, frontier=frontier,
-                                                    investment=xtra_dict['investment'])
+                            outputter.efficient_frontier(portfolio=portfolio, 
+                                                            frontier=frontier,
+                                                            investment=xtra_dict['investment'])
                         else:
-                            print(json.dumps(formatter.format_frontier(portfolio=portfolio, frontier=frontier, investment=xtra_dict['investment'])))
+                            print(json.dumps(formatter.format_frontier(portfolio=portfolio, 
+                                                                        frontier=frontier, 
+                                                                        investment=xtra_dict['investment'])))
 
                     if xtra_dict['save_file'] is not None:
-                        files.save_frontier(portfolio=portfolio, frontier=frontier,
+                        files.save_frontier(portfolio=portfolio, 
+                                            frontier=frontier,
                                             investment=xtra_dict['investment'], 
                                             file_name=xtra_dict['save_file'])
                 selected_function, required_length = cli_efficient_frontier, 2
@@ -343,25 +416,34 @@ def do_program():
             ### FUNCTION: Maximize Portfolio Return
             elif opt == formatter.FUNC_ARG_DICT['maximize_return']:
                 def cli_maximize_return():
-                    portfolio = Portfolio(tickers=main_args, start_date=xtra_dict['start_date'], 
-                                            end_date=xtra_dict['end_date'])
+                    portfolio = Portfolio(tickers=main_args, 
+                                            start_date=xtra_dict['start_date'], 
+                                            end_date=xtra_dict['end_date'],
+                                            method=xtra_dict['estimation'])
                     allocation = optimizer.maximize_portfolio_return(portfolio=portfolio)
 
                     if xtra_dict['suppress'] is None:
                         if xtra_dict['json'] is None:
-                            outputter.optimal_result(portfolio=portfolio, allocation=allocation, 
-                                                investment=xtra_dict['investment'])
+                            outputter.optimal_result(portfolio=portfolio, 
+                                                        allocation=allocation, 
+                                                        investment=xtra_dict['investment'])
                         else:
-                            print(json.dumps(formatter.format_allocation(allocation=allocation, portfolio=portfolio, investment=xtra_dict['investment'])))
+                            print(json.dumps(formatter.format_allocation(allocation=allocation, 
+                                                                            portfolio=portfolio, 
+                                                                            investment=xtra_dict['investment'])))
 
                     if xtra_dict['save_file'] is not None:
-                        files.save_allocation(allocation=allocation, portfolio=portfolio, 
-                                                file_name=xtra_dict['save_file'], investment=xtra_dict['investment'])
+                        files.save_allocation(allocation=allocation, 
+                                                portfolio=portfolio, 
+                                                file_name=xtra_dict['save_file'], 
+                                                investment=xtra_dict['investment'])
                 selected_function, required_length = cli_maximize_return, 2
 
             ### FUNCTION: Moving Averages of Logarithmic Returns
             elif opt == formatter.FUNC_ARG_DICT['moving_averages']:
                 def cli_moving_averages():
+                    # TODO: moving averages with estimation techniques
+                    # TODO: print results as json to screen and ability to save results
                     moving_averages = statistics.calculate_moving_averages(tickers=main_args, 
                                                                             start_date=xtra_dict['start_date'], 
                                                                             end_date=xtra_dict['end_date'])
@@ -375,7 +457,11 @@ def do_program():
             ### FUNCTION: Optimize Portfolio Variance/Volatility
             elif opt == formatter.FUNC_ARG_DICT['optimize_portfolio_variance']:
                 def cli_optimize_portfolio_variance():
-                    portfolio = Portfolio(tickers=main_args, start_date=xtra_dict['start_date'], end_date=xtra_dict['end_date'])
+                    portfolio = Portfolio(tickers=main_args, 
+                                            start_date=xtra_dict['start_date'], 
+                                            end_date=xtra_dict['end_date'],
+                                            method=xtra_dict['estimation'])
+
                     if xtra_dict['optimize_sharpe']:
                         allocation = optimizer.maximize_sharpe_ratio(portfolio=portfolio, target_return=xtra_dict['target'])
                     else:
@@ -394,7 +480,10 @@ def do_program():
             
             elif opt == formatter.FUNC_ARG_DICT['optimize_portfolio_conditional_var']:
                 def cli_optimize_conditional_value_at_risk():
-                    portfolio = Portfolio(tickers=main_args, start_date=xtra_dict['start_date'], end_date=xtra_dict['end_date'])
+                    portfolio = Portfolio(tickers=main_args, 
+                                            start_date=xtra_dict['start_date'], 
+                                            end_date=xtra_dict['end_date'],
+                                            method=xtra_dict['estimation'])
                     allocation = optimizer.optimize_conditional_value_at_risk(portfolio=portfolio,
                                                                                 prob=xtra_dict['probability'],
                                                                                 expiry=xtra_dict['expiry'],
@@ -409,46 +498,79 @@ def do_program():
                         files.save_allocation(allocation=allocation, portfolio=portfolio, file_name=xtra_dict['save_file'],
                                                 investment=xtra_dict['investment'])
                 selected_function, required_length = cli_optimize_conditional_value_at_risk, 2
+
             ### FUNCTION: Plot Dividend History With Linear Regression Model
             elif opt == formatter.FUNC_ARG_DICT['plot_dividends']:
                 def cli_plot_dividends():
                     dividends = services.get_dividend_history(ticker=main_args[0])
                     if xtra_dict['discount'] is None:
-                        xtra_dict['discount'] = markets.cost_of_equity(ticker=main_args[0])
-                    div_cashflow = Cashflow(sample=dividends, discount_rate=xtra_dict['discount'])
-                    plotter.plot_cashflow(ticker=main_args[0], cashflow=div_cashflow, show=True, 
+                        xtra_dict['discount'] = markets.cost_of_equity(ticker=main_args[0],
+                                                                        method=xtra_dict['estimation'])
+                    div_cashflow = Cashflow(sample=dividends, 
+                                            discount_rate=xtra_dict['discount'])
+                    plotter.plot_cashflow(ticker=main_args[0], 
+                                            cashflow=div_cashflow, show=True, 
                                             savefile=xtra_dict['save_file'])
                 selected_function, required_length, exact = cli_plot_dividends, 1, True
 
             ### FUNCTION: Plot Efficient Frontier
             elif opt == formatter.FUNC_ARG_DICT['plot_frontier']:
                 def cli_plot_frontier():
-                    portfolio = Portfolio(tickers=main_args, start_date=xtra_dict['start_date'], 
-                                            end_date=xtra_dict['end_date'])
-                    frontier = optimizer.calculate_efficient_frontier(portfolio=portfolio)
-                    plotter.plot_frontier(portfolio=Portfolio(main_args), frontier=frontier, show=True, 
+                    portfolio = Portfolio(tickers=main_args, 
+                                            start_date=xtra_dict['start_date'], 
+                                            end_date=xtra_dict['end_date'],
+                                            method=xtra_dict['estimation'])
+                    frontier = optimizer.calculate_efficient_frontier(portfolio=portfolio,
+                                                                        steps=xtra_dict['steps'])
+                    plotter.plot_frontier(portfolio=Portfolio(main_args), 
+                                            frontier=frontier, 
+                                            show=True, 
                                             savefile=xtra_dict['save_file'])
                 selected_function, required_length = cli_plot_frontier, 2
 
             ### FUNCTION: Plot Moving Averages of Logarithmic Returns
             elif opt == formatter.FUNC_ARG_DICT['plot_moving_averages']:
                 def cli_plot_moving_averages():
+                    # TODO: estimation techniques with moving averages
                     moving_averages = statistics.calculate_moving_averages(tickers=main_args, 
                                                                             start_date=xtra_dict['start_date'], 
                                                                             end_date=xtra_dict['end_date'])
                     periods = [settings.MA_1_PERIOD, settings.MA_2_PERIOD, settings.MA_3_PERIOD]
-                    plotter.plot_moving_averages(symbols=main_args, averages_output=moving_averages, periods=periods, 
+                    plotter.plot_moving_averages(symbols=main_args, 
+                                                    averages_output=moving_averages, 
+                                                    periods=periods, 
                                                     show=True, savefile=xtra_dict['save_file'])
                 selected_function, required_length = cli_plot_moving_averages, 1
 
+            ### FUNCTION: Plot Return QQ Series
+            elif opt == formatter.FUNC_ARG_DICT['plot_returns']:
+                def cli_plot_returns():
+                    asset_type = files.get_asset_type(symbol=main_args[0])
+                    prices = services.get_daily_price_history(ticker=main_args[0], 
+                                                                start_date=xtra_dict['start_date'],
+                                                                end_date=xtra_dict['end_date'],
+                                                                asset_type=asset_type)
+                    returns = statistics.get_sample_of_returns(prices, asset_type)
+                    qq_series = estimators.qq_series_for_sample(sample=returns)
+                    plotter.plot_qq_series(ticker=main_args[0], 
+                                            sample=qq_series, 
+                                            show=True, 
+                                            savefile=xtra_dict['save_file'])
+
+                selected_function, required_length, exact = cli_plot_returns, 1, True
+
             ### FUNCTION: Plot Risk-Return Profile
             elif opt == formatter.FUNC_ARG_DICT['plot_risk_profile']:
                 def cli_plot_risk_profile():
                     profiles = {}
                     for arg in main_args:
-                        profiles[arg]=statistics.calculate_risk_return(ticker=arg, start_date=xtra_dict['start_date'], 
-                                                                            end_date=xtra_dict['end_date'])
-                    plotter.plot_profiles(symbols=main_args, profiles=profiles, show=True, 
+                        profiles[arg]=statistics.calculate_risk_return(ticker=arg, 
+                                                                        start_date=xtra_dict['start_date'], 
+                                                                        end_date=xtra_dict['end_date'],
+                                                                        method=xtra_args['estimation'])
+                    plotter.plot_profiles(symbols=main_args, 
+                                            show=True,
+                                            profiles=profiles, 
                                             savefile=xtra_dict['save_file'], 
                                             subtitle=helper.format_date_range(start_date=xtra_dict['start_date'], 
                                                                                 end_date=xtra_dict['end_date']))
@@ -461,7 +583,8 @@ def do_program():
                     start_date_string = helper.date_to_string(xtra_dict['start_date'])
                     yield_curve[start_date_string] = []
                     for maturity in static.keys['YIELD_CURVE']:
-                        rate = services.get_daily_interest_history(maturity=maturity, start_date=xtra_dict['start_date'],
+                        rate = services.get_daily_interest_history(maturity=maturity, 
+                                                                    start_date=xtra_dict['start_date'],
                                                                     end_date=xtra_dict['start_date'])
                         yield_curve[start_date_string].append(rate[start_date_string])
                 
@@ -474,7 +597,8 @@ def do_program():
                 def cli_price_history():
                     all_prices = {}
                     for arg in main_args:
-                        prices = services.get_daily_price_history(ticker=arg, start_date=xtra_dict['start_date'],
+                        prices = services.get_daily_price_history(ticker=arg, 
+                                                                    start_date=xtra_dict['start_date'],
                                                                     end_date=xtra_dict['end_date'])
                         all_prices[arg] = {}
                         for date in prices:
@@ -538,16 +662,24 @@ def do_program():
                 def cli_risk_return():
                     profiles = {}
                     for arg in main_args:
-                        profiles[arg] = statistics.calculate_risk_return(ticker=arg, start_date=xtra_dict['start_date'], 
+                        profiles[arg] = statistics.calculate_risk_return(ticker=arg, 
+                                                                            method = xtra_dict['estimation'],
+                                                                            start_date=xtra_dict['start_date'], 
                                                                             end_date=xtra_dict['end_date'])
-                        profiles[arg]['sharpe_ratio'] = markets.sharpe_ratio(ticker=arg, start_date=xtra_dict['start_date'],
+                        profiles[arg]['sharpe_ratio'] = markets.sharpe_ratio(ticker=arg, 
+                                                                            start_date=xtra_dict['start_date'],
                                                                             end_date=xtra_dict['end_date'], 
-                                                                            ticker_profile=profiles[arg])
-                        profiles[arg]['asset_beta'] = markets.market_beta(ticker=arg, start_date=xtra_dict['start_date'],
+                                                                            ticker_profile=profiles[arg],
+                                                                            method=xtra_dict['estimation'])
+                        profiles[arg]['asset_beta'] = markets.market_beta(ticker=arg, 
+                                                                            start_date=xtra_dict['start_date'],
                                                                             end_date=xtra_dict['end_date'],
-                                                                            ticker_profile=profiles[arg])
-                        profiles[arg]['equity_cost'] = markets.cost_of_equity(ticker=arg, start_date=xtra_dict['start_date'],
-                                                                            end_date=xtra_dict['end_date'])
+                                                                            ticker_profile=profiles[arg],
+                                                                            method=xtra_dict['estimation'])
+                        profiles[arg]['equity_cost'] = markets.cost_of_equity(ticker=arg, 
+                                                                            start_date=xtra_dict['start_date'],
+                                                                            end_date=xtra_dict['end_date'], 
+                                                                            method=xtra_dict['estimation'])
                     
                     if xtra_dict['suppress'] is None:
                         if xtra_dict['json'] is None:
@@ -573,8 +705,10 @@ def do_program():
                 def cli_sharpe_ratio():
                     all_results = {}
                     for arg in main_args:
-                        result = markets.sharpe_ratio(ticker=arg, start_date=xtra_dict['start_date'],
-                                                        end_date=xtra_dict['end_date'])
+                        result = markets.sharpe_ratio(ticker=arg, 
+                                                        start_date=xtra_dict['start_date'],
+                                                        end_date=xtra_dict['end_date'],
+                                                        method=xtra_dict['estimation'])
                         all_results[arg]=result
                         
                         if print_format_to_screen(xtra_dict):
@@ -591,7 +725,6 @@ def do_program():
 
             ### FUNCTION: Get Latest Economic Statistic
             elif opt == formatter.FUNC_ARG_DICT["statistic"]:
-                # TODO: implement start and end date and print eac.
                 def cli_statistic():
                     all_stats = {}
                     for stat in main_args:
@@ -612,12 +745,14 @@ def do_program():
                 def cli_statistic_history():
                     all_stats = {}
                     for arg in main_args:
-                        stats = services.get_daily_fred_history(symbol=arg, start_date=xtra_dict['start_date'],
-                                                            end_date=xtra_dict['end_date'])
+                        stats = services.get_daily_fred_history(symbol=arg, 
+                                                                start_date=xtra_dict['start_date'],
+                                                                end_date=xtra_dict['end_date'])
                         all_stats[arg] = stats
                         if print_format_to_screen(xtra_dict):
                             for date in stats:
-                                outputter.scalar_result(calculation=f'{arg}({date})', result=stats[date], 
+                                outputter.scalar_result(calculation=f'{arg}({date})', 
+                                                            result=stats[date], 
                                                             currency=False) 
 
                     if print_json_to_screen(xtra_dict):
@@ -655,9 +790,11 @@ def do_program():
                 outputter.help_msg()
             
             if selected_function is not None:
-                validate_function_usage(selection=opt, args=main_args, 
-                                        wrapper_function=selected_function, 
-                                        required_length=required_length, exact=exact)
+                validate_function_usage(selection=opt, 
+                                            args=main_args, 
+                                            wrapper_function=selected_function, 
+                                            required_length=required_length, 
+                                            exact=exact)
     
             if print_format_to_screen(xtra_dict):
                 outputter.print_line()
@@ -666,8 +803,4 @@ def do_program():
         outputter.help_msg()
 
 if __name__ == "__main__": 
-    # TODO: check for API key and if none exists, prompt user to define ALPHA_VANTAGE_KEY, QUANDL_KEY and
-    #       IEX_KEY environment variables. Then exit. 
-    # TODO: possibly create PyQt widget to prompt user to enter credentials and save them to creds.json
-    #       in /data/common/
     do_program()
\ No newline at end of file
