@@ -1,4 +1,20 @@
-import itertools, time, requests
+# This file is part of scrilla: https://github.com/chinchalinchin/scrilla.
+
+# scrilla is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 3
+# as published by the Free Software Foundation.
+
+# scrilla is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with scrilla.  If not, see <https://www.gnu.org/licenses/>
+# or <https://github.com/chinchalinchin/scrilla/blob/develop/main/LICENSE>.
+
+import itertools, time, datetime, requests
+from typing import Union
 
 from scrilla import settings, errors, cache, static
 import scrilla.util.outputter as outputter
@@ -10,7 +26,7 @@ class StatManager():
     """
     Description
     -----------
-        StatManager is an interface between the application and the external services that hydrate it with financial statistics data. This class gets instantiated on the level of the scrilla.services module with the value defined in `scrilla.settings.STAT_MANAGER`. This value is in turn defined by the value of the `STAT_MANAGER` environment variable. This value determines how the url is constructed, which API credentials get appended to the external query and the keys used to parse the response JSON container the statistical data.
+        StatManager is an interface between the application and the external services that hydrate it with financial statistics data. This class gets instantiated on the level of the `scrilla.services` module with the value defined in `scrilla.settings.STAT_MANAGER`. This value is in turn defined by the value of the `STAT_MANAGER` environment variable. This value determines how the url is constructed, which API credentials get appended to the external query and the keys used to parse the response JSON container the statistical data.
     """
     def __init__(self, type):
         self.type = type
@@ -18,27 +34,90 @@ class StatManager():
             self.service_map = static.keys["SERVICES"]["STATISTICS"]["QUANDL"]["MAP"]
 
     def is_quandl(self):
+        """
+        Returns
+        -------
+        bool \n
+            `True` if this instace of `StatManager` is a Quandl interface. `False` otherwise. \n\n
+
+        .. notes::
+            * This is for use within the class and probably won't need to be accessed outside of it. `StatManager` is intended to hide the data implementation from the rest of the library, i.e. it is ultimately agnostic about where the data comes where. It should never need to know `StatManger` is a Quandl interface. Just in case the library ever needs to populate its data from another source. \n \n
+
+        """
         if self.type == static.keys['SERVICES']['STATISTICS']['QUANDL']['MANAGER']:
             return True
         return False
 
-    def construct_query(self, start_date, end_date):
+    def construct_query(self, start_date : datetime.date, end_date: datetime.date) -> str:
+        """
+        Description
+        -----------
+        Constructs and formats the query parameters for the external statistics service. Note, this method appends the API key to the query. Be careful with the returned value.
+
+        Parameters
+        ----------
+        1. start_date : datetime.date \n 
+            Start date of historical sample to be retrieved. \n \n
+        2. end_date : datetime.date \n 
+            End date of historical sample to be retrieved. \n \n
+
+        Raises
+        ------
+        1. scrilla.errors.ConfigurationError \n
+            If the `STAT_MANAGER` hasn't been set through an enviornment variable, this error will be thrown. \n \n
+
+        Returns
+        -------
+        str \n
+            The formatted query for the specific service defined by `self.type`. \n \n
+
+        """
         query = ""
-        if end_date is not None:
-            end_string = helper.date_to_string(end_date)
-            query += f'&{self.service_map["PARAMS"]["END"]}={end_string}' 
+        if self.is_quandl():
+
+            if end_date is not None:
+                end_string = helper.date_to_string(end_date)
+                query += f'&{self.service_map["PARAMS"]["END"]}={end_string}' 
+
+            if start_date is not None:
+                start_string = helper.date_to_string(start_date)
+                query += f'&{self.service_map["PARAMS"]["START"]}={start_string}'
+        
+            logger.debug(f'Quandl query (w/o key) = {query}')
+
+            if query:
+                return f'{query}&{self.service_map["PARAMS"]["KEY"]}={settings.Q_KEY}'
+
+            return f'{self.service_map["PARAMS"]["KEY"]}={settings.Q_KEY}'
+        raise errors.ConfigurationError('No STAT_MANAGER found in the parsed environment settings')
 
-        if start_date is not None:
-            start_string = helper.date_to_string(start_date)
-            query += f'&{self.service_map["PARAMS"]["START"]}={start_string}'
-    
-        logger.debug(f'Quandl query (w/o key) = {query}')
-        if query:
-            return f'{query}&{self.service_map["PARAMS"]["KEY"]}={settings.Q_KEY}'
-        return f'{self.service_map["PARAMS"]["KEY"]}={settings.Q_KEY}'
 
 
     def construct_stat_url(self, symbol, start_date, end_date):
+        """
+        Description
+        -----------
+        Constructs the full URL path for the external statistics service. Note, this method will return the URL with an API key appended as a query parameter. Be careful with the returned value. \n\n
+
+        Parameters
+        ----------
+        1. symbol: str \n
+            Symbol representing the statistical series to be retrieved. List of allowable symbols can be found here: https://data.nasdaq.com/data/FRED-federal-reserve-economic-data
+        2. start_date : datetime.date \n 
+            Start date of historical sample to be retrieved. \n \n
+        3. end_date : datetime.date \n 
+            End date of historical sample to be retrieved. \n \n
+ 
+        Raises
+        ------
+        1. scrilla.errors.ConfigurationError \n
+            If the `STAT_MANAGER` hasn't been set through an enviornment variable, this error will be thrown. \n \n
+
+        Returns
+        -------
+        str \n
+            The formatted URL for the specific statistics service defined by `self.type`. \n \n
+        """
         if self.is_quandl():
             url = f'{settings.Q_URL}/{self.service_map["PATHS"]["FRED"]}/{symbol}?'
             url += self.construct_query(start_date, end_date)
@@ -46,6 +125,31 @@ class StatManager():
         raise errors.ConfigurationError('No STAT_MANAGER found in the parsed environment settings')
 
     def construct_interest_url(self,start_date, end_date):
+        """
+        Description
+        -----------
+        Constructs the full URL path for the external interest rate service. Note, this method will return the URL with an API key appended as a query parameter. Be careful with the returned value. \n\n
+
+        Parameters
+        ----------
+        2. start_date : datetime.date \n 
+            Start date of historical sample to be retrieved. \n \n
+        3. end_date : datetime.date \n 
+            End date of historical sample to be retrieved. \n \n
+ 
+        Raises
+        ------
+        1. scrilla.errors.ConfigurationError \n
+            If the `STAT_MANAGER` hasn't been set through an enviornment variable, this error will be thrown. \n \n
+
+        Returns
+        -------
+        str \n
+            The formatted URL for the specific interest rate service defined by `self.type`. \n \n
+
+        .. notes::
+            * The URL returned by this method will always contain a query for a historical range of US Treasury Yields, i.e. this method is specifically for queries involving the "Risk-Free" (right? right? *crickets*) Yield Curve. 
+        """
         if self.is_quandl():
             url = f'{settings.Q_URL}/{self.service_map["PATHS"]["YIELD"]}?'
             url += self.construct_query(start_date=start_date, end_date=end_date)
@@ -96,7 +200,6 @@ class StatManager():
         
         raise errors.ConfigurationError('No STAT_MANAGER found in the parsed environment settings')
 
-
 class DividendManager():
     
     def __init__(self, type):
@@ -147,6 +250,7 @@ class PriceManager():
 
         Returns
         -------
+        str \n
             The URL with the authenticated query appended, i.e. with the service's API key injected into the parameters. Be careful not to expose the return value of this function! \n \n
 
         Raises
@@ -281,6 +385,8 @@ class PriceManager():
     def slice_prices(self, start_date, end_date, asset_type, prices):
         # NOTE: only really needed for `alpha_vantage` responses so far, due to the fact AlphaVantage either returns everything or 100 days or prices.
         if self.type == static.keys['SERVICES']['PRICES']['ALPHA_VANTAGE']['MANAGER']:
+            service_map = static.keys['SERVICES']['PRICES']['ALPHA_VANTAGE']['MAP']
+
             # NOTE: Remember AlphaVantage is ordered current to earliest. END_INDEX is 
             # actually the beginning of slice and START_INDEX is actually end of slice. 
             try:
@@ -304,17 +410,19 @@ class PriceManager():
     def parse_price_from_date(self, prices, date, asset_type, which_price):
         try:
             if self.type== 'alpha_vantage':
+                service_map = static.keys['SERVICES']['PRICES']['ALPHA_VANTAGE']['MAP']
+
                 if asset_type == static.keys['ASSETS']['EQUITY']:
                     if which_price == static.keys['PRICES']['CLOSE']:
-                        return prices[date][settings.AV_RES_EQUITY_CLOSE_PRICE]
+                        return prices[date][service_map['KEYS']['EQUITY']['CLOSE']]
                     if which_price == static.keys['PRICES']['OPEN']:
-                        return prices[date][settings.AV_RES_EQUITY_OPEN_PRICE]
+                        return prices[date][service_map['KEYS']['EQUITY']['CLOSE']]
 
                 elif asset_type == static.keys['ASSETS']['CRYPTO']:
                     if which_price == static.keys['PRICES']['CLOSE']:
-                        return prices[date][settings.AV_RES_CRYPTO_CLOSE_PRICE]
+                        return prices[date][service_map['KEYS']['CRYPTO']['CLOSE']]
                     if which_price == static.keys['PRICES']['OPEN']:
-                        return prices[date][settings.AV_RES_CRYPTO_OPEN_PRICE]
+                        return prices[date][service_map['KEYS']['CRYPTO']['OPEN']]
             
             raise errors.InputValidationError(f'Verify {asset_type}, {which_price} are allowable values')
 
@@ -328,51 +436,34 @@ div_manager = DividendManager(settings.DIV_MANAGER)
 price_cache = cache.PriceCache()        
 interest_cache = cache.InterestCache()
 
-def get_daily_price_history(ticker, start_date=None, end_date=None, asset_type=None):
+def get_daily_price_history(ticker: str, start_date : datetime.date=None, 
+                            end_date: datetime.date =None, asset_type: str=None) -> list:
     """
-    Description
-    -----------
     Wrapper around external service request for price data. Relies on an instance of `PriceManager` configured by `settings.PRICE_MANAGER` value, which in turn is configured by the `PRICE_MANAGER` environment variable, to hydrate with data. \n \n
     
-    Before deferring to the `PriceManager` and letting it call the external service, however, this function checks if response is in local cache. If the response is not in the cache, it will pass the request off to `PriceManager` and then save the response in the cache so subsequent calls to the function can bypass the service request. Used to prevent excessive external HTTP requests and improve the performance of the application. Other parts of the program should interface with the external price data services through this function to utilize the cache functionality.  \n \n
+    Before deferring to the `PriceManager` and letting it call the external service, however, this function checks if response is in local cache. If the response is not in the cache, it will pass the request off to `PriceManager` and then save the response in the cache so subsequent calls to the function can bypass the service request. Used to prevent excessive external HTTP requests and improve the performance of the application. Other parts of the program should interface with the external price data services through this function to utilize the cache functionality.
 
     Parameters
     ----------
-    1. ticker :  str  \n
-        Required. Ticker symbol corresponding to the price history to be retrieved. \n \n
-    2. start_date : datetime.date \n 
-        Optional. Start date of price history. Defaults to None. If `start_date is None`, the calculation is made as if the `start_date` were set to 100 trading days ago. If `get_asset_type(ticker)=="crypto"`, this includes weekends and holidays. If `get_asset_type(ticker)=="equity"`, this excludes weekends and holidays. \n \n
-    3. end_date : datetime.date \n 
-        Optional End date of price history. Defaults to None. If `end_date is None`, the calculation is made as if the `end_date` were set to today. If `get_asset_type(ticker)=="crypto"`, this means today regardless. If `get_asset_type(ticker)=="equity"`, this excludes weekends and holidays so that `end_date` is set to the previous business date. \n \n
-    4. asset_type : string \n
-        Optional. Asset type of the ticker whose history is to be retrieved. Used to prevent excessive calls to IO and list searching. `asset_type` is determined by comparing the ticker symbol `ticker` to a large static list of ticker symbols maintained in installation directory's /data/static/ subdirectory, which can slow the program down if the file is constantly accessed and lots of comparison are made against it. Once an `asset_type` is calculated, it is best to preserve it in the process environment somehow, so this function allows the value to be passed in. If no value is detected, it will make a call to the aforementioned directory and parse the file to determine to the `asset_type`. Asset types are statically accessible through the `scrilla.static.keys['ASSETS']` dictionary. \n \n
-
-    Raises
-    ------
-    1. scrilla.errors.InputValidationError \n
-        If the arguments inputted into the function fail to exist within the domain the function, this error will be thrown.
-    2. scrilla.errors.APIResponseError \n
-        If the external service rejects the request for price data, whether because of rate limits or some other factor, the function will raise this exception.
-    3. KeyError \n
-        If the inputted or validated dates do not exist in the price history, a KeyError will be thrown. This could be due to the equity not having enough price history, i.e. it started trading a month ago and doesn't have 100 days worth of prices yet, or some other anomalous event in an equity's history. 
-    4. errors.ConfigurationError \n
-        If one of the settings is improperly configured or one of the environment variables was unable to be parsed from the environment, this error will be thrown. \n \n
+    1. **ticker** :  ``str``
+        Ticker symbol corresponding to the price history to be retrieved.
+    2. **start_date** : ``datetime.date`` 
+        *Optional*. Start date of price history. Defaults to None. If `start_date is None`, the calculation is made as if the `start_date` were set to 100 trading days ago. If `scrilla.files.get_asset_type(ticker)==scrill.static.keys['ASSETS']['CRYPTO']`, this includes weekends and holidays. If `scrilla.files.get_asset_type(ticker)==scrilla.static.keys['ASSETS']['EQUITY']`, this excludes weekends and holidays.
+    3. **end_date** : ``datetime.date``
+        Optional End date of price history. Defaults to None. If `end_date is None`, the calculation is made as if the `end_date` were set to today. If `scrilla.files.get_asset_type(ticker)==scrill.static.keys['ASSETS']['CRYPTO']`, this means today regardless. If `scrilla.files.get_asset_type(ticker)==scrilla.static.keys['ASSETS']['EQUITY']`, this excludes weekends and holidays so that `end_date` is set to the previous business date. 
+    4. **asset_type** : ``string``
+        *Optional*. Asset type of the ticker whose history is to be retrieved. Used to prevent excessive calls to IO and list searching. `asset_type` is determined by comparing the ticker symbol `ticker` to a large static list of ticker symbols maintained in installation directory's /data/static/ subdirectory, which can slow the program down if the file is constantly accessed and lots of comparison are made against it. Once an `asset_type` is calculated, it is best to preserve it in the process environment somehow, so this function allows the value to be passed in. If no value is detected, it will make a call to the aforementioned directory and parse the file to determine to the `asset_type`. Asset types are statically accessible through the `scrilla.static.keys['ASSETS']` dictionary.
 
     Returns
     ------
-    { 'date' (str) : { 'open': value (str), 'close': value (str) }, 'date' (str): { 'open' : value (str), 'close' : value(str) }, ... }
-        Dictionary with date strings formatted `YYYY-MM-DD` as keys and a nested dictionary containing the 'open' and 'close' price as values. Ordered from latest to earliest. \n \n
+    ``dict``: `{ 'date' (str) : { 'open': value (str), 'close': value (str) }, 'date' (str): { 'open' : value (str), 'close' : value(str) }, ... }`, dictionary with date strings formatted `YYYY-MM-DD` as keys and a nested dictionary containing the 'open' and 'close' price as values. Ordered from latest to earliest.
     
-    Notes
-    -----
-    1. The default analysis period, if no `start_date` and `end_date` are specified, is determined by the *DEFAULT_ANALYSIS_PERIOD" variable in the `settings,py` file. The hardcoded value of this setting is 100. Should probably put this variable into the enviroment in the future and allow user to configure it. \n \n
+    .. notes::
+        * The default analysis period, if no `start_date` and `end_date` are specified, is determined by the *DEFAULT_ANALYSIS_PERIOD** variable in the `settings,py` file. The default value of this variable is 100.
     """
-    try:
-        asset_type = errors.validate_asset_type(ticker, asset_type)
-        start_date, end_date = errors.validate_dates(start_date, end_date, asset_type)
-    except errors.InputValidationError as ive:
-        raise ive
-
+    asset_type = errors.validate_asset_type(ticker, asset_type)
+    start_date, end_date = errors.validate_dates(start_date, end_date, asset_type)
+    
     prices = price_cache.filter_price_cache(ticker=ticker, start_date=start_date, end_date=end_date)
 
     if prices is not None and helper.date_to_string(end_date) in prices.keys() and (
@@ -383,15 +474,11 @@ def get_daily_price_history(ticker, start_date=None, end_date=None, asset_type=N
             and (helper.days_between(start_date, end_date) + 1) == len(prices))
     ):
         return prices
+
     if prices is not None:
         logger.debug(f'Cached {ticker} prices are out of date, passing request off to external service')
         
-    try:
-        prices = price_manager.get_prices(ticker=ticker,start_date=start_date, end_date=end_date, asset_type=asset_type)
-    except errors.APIResponseError as api:
-        raise api
-    except errors.InputValidationError as ive:
-        raise ive
+    prices = price_manager.get_prices(ticker=ticker,start_date=start_date, end_date=end_date, asset_type=asset_type)
 
     parsed_prices ={}
     for date in prices:
@@ -404,7 +491,7 @@ def get_daily_price_history(ticker, start_date=None, end_date=None, asset_type=N
 
     return parsed_prices
     
-def get_daily_price_latest(ticker, asset_type=None):
+def get_daily_price_latest(ticker: str, asset_type: str=None) -> float:
     """
     Description
     -----------
@@ -423,14 +510,12 @@ def get_daily_price_latest(ticker, asset_type=None):
         return prices[first_element][static.keys['PRICES']['OPEN']]
     return None
 
-def get_daily_fred_history(symbol, start_date=None, end_date=None):
+def get_daily_fred_history(symbol: str, start_date: datetime.date=None, end_date: datetime.date=None) -> list:
     """
     Description
     -----------
     Wrapper around external service request for financial statistics data constructed by the Federal Reserve Economic Data. Relies on an instance of `StatManager` configured by `settings.STAT_MANAGER` value, which in turn is configured by the `STAT_MANAGER` environment variable, to hydrate with data. \n \n
     
-    Before deferring to the `StatManager` and letting it call the external service, however, this function checks if response is in local cache. If the response is not in the cache, it will pass the request off to `StatManager` and then save the response in the cache so subsequent calls to the function can bypass the service request. Used to prevent excessive external HTTP requests and improve the performance of the application. Other parts of the program should interface with the external statistics data services through this function to utilize the cache functionality.  \n \n
-
     Parameters
     ----------
     1. symbol: str \n 
@@ -456,9 +541,8 @@ def get_daily_fred_history(symbol, start_date=None, end_date=None):
     { 'date' (str) :  value (str),  'date' (str):  value (str), ... }
         Dictionary with date strings formatted `YYYY-MM-DD` as keys and the statistic on that date as the corresponding value. \n \n
 
-    Notes
-    -----
-    1. Most financial statistics are not reported on weekends or holidays, so the `asset_type` for financial statistics is functionally equivalent to equities, at least as far as date calculations are concerned. The dates inputted into this function are validated as if they were labelled as equity `asset_types` for this reason.
+    .. notes::
+        * Most financial statistics are not reported on weekends or holidays, so the `asset_type` for financial statistics is functionally equivalent to equities, at least as far as date calculations are concerned. The dates inputted into this function are validated as if they were labelled as equity `asset_types` for this reason.
 
     Note: there are probably cases where this function will break because the statistical data isn't reported on a daily basis. In fact, there are tons of cases where this doesn't work...
 
@@ -477,16 +561,14 @@ def get_daily_fred_history(symbol, start_date=None, end_date=None):
 
     return stats
 
-def get_daily_fred_latest(symbol):
+def get_daily_fred_latest(symbol: str) -> Union[float,None]:
     """
-    Description
-    -----------
-    Returns the latest value for the inputted statistic symbol. \n \n
+    Returns the latest value for the inputted statistic symbol.
 
     Parameters
     ----------
-    1. statistic: str \n 
-        Required. Symbol representing the statistc whose value it to be retrieved. \n \n
+    1. **symbol**: str
+        Symbol representing the statistic whose value it to be retrieved.
     """
     stats_history = get_daily_fred_history(symbol=symbol)
     if stats_history is not None:
@@ -494,42 +576,39 @@ def get_daily_fred_latest(symbol):
         return stats_history[first_element]
     return None
 
-def get_daily_interest_history(maturity, start_date=None, end_date=None):
+def get_daily_interest_history(maturity: str, start_date: datetime.date=None, end_date: datetime.date=None) -> list:
     """
-    Description
-    -----------
-    Wrapper around external service request for financial statistics data constructed by the Federal Reserve Economic Data. Relies on an instance of `StatManager` configured by `settings.STAT_MANAGER` value, which in turn is configured by the `STAT_MANAGER` environment variable, to hydrate with data. \n \n
+    Wrapper around external service request for US Treasury Yield Curve data. Relies on an instance of `StatManager` configured by `settings.STAT_MANAGER` value, which in turn is configured by the `STAT_MANAGER` environment variable, to hydrate with data.
     
-    Before deferring to the `StatManager` and letting it call the external service, however, this function checks if response is in local cache. If the response is not in the cache, it will pass the request off to `StatManager` and then save the response in the cache so subsequent calls to the function can bypass the service request. Used to prevent excessive external HTTP requests and improve the performance of the application. Other parts of the program should interface with the external statistics data services through this function to utilize the cache functionality.  \n \n
+    Before deferring to the `StatManager` and letting it call the external service, however, this function checks if response is in local cache. If the response is not in the cache, it will pass the request off to `StatManager` and then save the response in the cache so subsequent calls to the function can bypass the service request. Used to prevent excessive external HTTP requests and improve the performance of the application. Other parts of the program should interface with the external statistics data services through this function to utilize the cache functionality. 
 
     Parameters
     ----------
-    1. Maturity: str \n 
-        Required. Maturity of the US Treasury for which the interest rate will be retrieved. List of allowable values can in `scrilla.stats.keys['SERVICES']['STATISTICS']['QUANDL']['MAP']['YIELD_CURVE']` \n \n
-     2. start_date : datetime.date \n 
-        Optional. Start date of price history. Defaults to None. If `start_date is None`, the calculation is made as if the `start_date` were set to 100 trading days ago. This excludes weekends and holidays. \n \n
-    3. end_date : datetime.date \n 
-        Optional End date of price history. Defaults to None. If `end_date is None`, the calculation is made as if the `end_date` were set to today. This excludes weekends and holidays so that `end_date` is set to the last previous business date. \n \n
+    1. **maturity** : ``str``
+        Maturity of the US Treasury for which the interest rate will be retrieved. List of allowable values can in `scrilla.stats.keys['SERVICES']['STATISTICS']['QUANDL']['MAP']['YIELD_CURVE']`
+    2. **start_date** : ``datetime.date``
+        *Optional*. Start date of price history. Defaults to None. If `start_date is None`, the calculation is made as if the `start_date` were set to 100 trading days ago. This excludes weekends and holidays.
+    3. **end_date** : ``datetime.date``` 
+        *Optional*. End date of price history. Defaults to None. If `end_date is None`, the calculation is made as if the `end_date` were set to today. This excludes weekends and holidays so that `end_date` is set to the last previous business date.
     
     Raises
     ------
-    1. scrilla.errors.InputValidationError \n
+    1. **scrilla.errors.InputValidationError**
         If the arguments inputted into the function fail to exist within the domain the function, this error will be thrown.
-    2. scrilla.errors.APIResponseError \n
+    2. **scrilla.errors.APIResponseError**
         If the external service rejects the request for price data, whether because of rate limits or some other factor, the function will raise this exception.
-    3. KeyError \n
+    3. **KeyError**
         If the inputted or validated dates do not exist in the price history, a KeyError will be thrown. This could be due to the equity not having enough price history, i.e. it started trading a month ago and doesn't have 100 days worth of prices yet, or some other anomalous event in an equity's history. 
-    4. scrilla.errors.ConfigurationError \n
-        If one of the settings is improperly configured or one of the environment variables was unable to be parsed from the environment, this error will be thrown. \n \n
+    4. **scrilla.errors.ConfigurationError**
+        If one of the settings is improperly configured or one of the environment variables was unable to be parsed from the environment, this error will be thrown.
 
     Returns
     ------
-    { 'date' (str) :  value (str),  'date' (str):  value (str), ... }
-        Dictionary with date strings formatted `YYYY-MM-DD` as keys and the interest on that date as the corresponding value. \n \n
+    ``dict`` : `{ 'date' :  value ,  'date':  value , ... }`
+        Dictionary with date strings formatted `YYYY-MM-DD` as keys and the interest on that date as the corresponding value.
 
-    Notes
-    -----
-    1. Yield rates are not reported on weekends or holidays, so the `asset_type` for interest is functionally equivalent to equities, at least as far as date calculations are concerned. The dates inputted into this function are validated as if they were labelled as equity `asset_types` for this reason.
+    .. notes::
+        * Yield rates are not reported on weekends or holidays, so the `asset_type` for interest is functionally equivalent to equities, at least as far as date calculations are concerned. The dates inputted into this function are validated as if they were labelled as equity `asset_types` for this reason.
     """
     try:
         start_date,end_date=errors.validate_dates(start_date=start_date, end_date=end_date, asset_type=static.keys['ASSETS']['EQUITY'])
@@ -537,7 +616,7 @@ def get_daily_interest_history(maturity, start_date=None, end_date=None):
         raise ive
 
     rates = None
-    rates = interest_cache.filter_stat_cache(maturity, start_date=start_date, end_date=end_date)
+    rates = interest_cache.filter_interest_cache(maturity, start_date=start_date, end_date=end_date)
 
         # TODO: this only works when stats are reported daily and that the latest date in the dataset is actually end_date.
     if rates is not None and helper.date_to_string(end_date) in rates.keys() \
@@ -559,68 +638,45 @@ def get_daily_interest_history(maturity, start_date=None, end_date=None):
 
     return rates
 
-def get_daily_interest_latest(maturity):
+def get_daily_interest_latest(maturity: str) -> float:
     """
-    Description
-    -----------
-    Returns the latest interest rate for the inputted US Treasury maturity. \n \n
+    Returns the latest interest rate for the inputted US Treasury maturity.
 
     Parameters
     ----------
-    1. maturity: str \n 
-        Required. Maturity of the US Treasury security whose interest rate is to be retrieved. \n \n
+    1. **maturity**: ``str``
+        Maturity of the US Treasury security whose interest rate is to be retrieved. Allowable values accessible through `static.keys['YIELD_CURVE']
     """
     end_date = helper.get_last_trading_date()
-    start_date = end_date
+    start_date = helper.decrement_date_by_business_days(end_date, 1)
     interest_history = get_daily_interest_history(maturity=maturity, start_date=start_date, end_date=end_date)
     if interest_history is not None:
         first_element = helper.get_first_json_key(interest_history)
         return interest_history[first_element]
     return None
 
-def get_dividend_history(ticker):
+def get_dividend_history(ticker: str) -> dict:
     """
-    Description
-    -----------
-    Wrapper around external service request for dividend payment data. Relies on an instance of `DivManager` configured by `settings.DIV_MANAGER` value, which in turn is configured by the `DIV_MANAGER` environment variable, to hydrate with data. \n \n
+    Wrapper around external service request for dividend payment data. Relies on an instance of `DivManager` configured by `settings.DIV_MANAGER` value, which in turn is configured by the `DIV_MANAGER` environment variable, to hydrate with data.
     
     Note, since dividend payments do not occur every day (if only), dividend amounts do not get cached, as there is no nice way to determine on a given day whether or not a payment should have been made, and thus to determine whether or not the cache is out of date. In other words, you can't look at today's date and the date of the last payment in the cache and determine based solely on the dates whether or not the cache is outdated. 
 
     Parameters
     ----------
-    1. ticker : str \n 
-        Required. Ticker symbol of the equity whose dividend history is to be retrieved. \n \n 
+    1. **ticker** : ``str`` 
+        Ticker symbol of the equity whose dividend history is to be retrieved.
     
-    Raises
-    ------
-    1. scrilla.errors.InputValidationError \n
-        If the arguments inputted into the function fail to exist within the domain the function, this error will be thrown.
-    2. scrilla.errors.APIResponseError \n
-        If the external service rejects the request for price data, whether because of rate limits or some other factor, the function will raise this exception.
-    3. KeyError \n
-        If the inputted or validated dates do not exist in the price history, a KeyError will be thrown. This could be due to the equity not having enough price history, i.e. it started trading a month ago and doesn't have 100 days worth of prices yet, or some other anomalous event in an equity's history. 
-    4. errors.ConfigurationError \n
-        If one of the settings is improperly configured or one of the environment variables was unable to be parsed from the environment, this error will be thrown. \n \n
-
     Returns
     ------
-    { 'date' (str) :  amount (str),  'date' (str):  amount (str), ... }
+    ``list`` : `{ 'date' (str) :  amount (str),  'date' (str):  amount (str), ... }`
         Dictionary with date strings formatted `YYYY-MM-DD` as keys and the dividend payment amount on that date as the corresponding value. \n \n
     """
-    try:
-        logger.debug(f'Retrieving {ticker} dividends from service')  
-        divs = div_manager.get_dividends(ticker=ticker)
-    except errors.APIResponseError as api:
-        raise api
-    except errors.InputValidationError as ive:
-        raise ive
-    
+    logger.debug(f'Retrieving {ticker} dividends from service')  
+    divs = div_manager.get_dividends(ticker=ticker)
     return divs
 
-def get_risk_free_rate():
+def get_risk_free_rate() -> float:
     """
-    Description
-    -----------
-    Returns the risk free rate, defined as the annualized yield on a specific US Treasury duration, as a decimal. The US Treasury yield used as a proxy for the risk free rate is defined in the `settings.py` file and is configured through the RISK_FREE environment variable. \n \n 
+    Returns the risk free rate, defined as the annualized yield on a specific US Treasury duration, as a decimal. The US Treasury yield used as a proxy for the risk free rate is defined in the `settings.py` file and is configured through the RISK_FREE environment variable.
     """
     return get_daily_interest_latest(maturity=settings.RISK_FREE_RATE)/100
