@@ -1,5 +1,19 @@
+# This file is part of scrilla: https://github.com/chinchalinchin/scrilla.
+
+# scrilla is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 3
+# as published by the Free Software Foundation.
+
+# scrilla is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with scrilla.  If not, see <https://www.gnu.org/licenses/>
+# or <https://github.com/chinchalinchin/scrilla/blob/develop/main/LICENSE>.
+
 import os, sys, dotenv, json
-from scrilla import static
 
 APP_DIR = os.path.dirname(os.path.abspath(__file__))
 PROJECT_DIR = os.path.dirname(APP_DIR)
@@ -7,7 +21,8 @@ ROOT_DIR = os.path.dirname(PROJECT_DIR)
 
 sys.path.append(APP_DIR)
 
-import util.outputter as outputter
+
+import scrilla.util.outputter as outputter
 
 class APIKeyError(Exception):
     pass
@@ -158,6 +173,10 @@ RISK_FREE_RATE=os.environ.setdefault("RISK_FREE", "TEN_YEAR").strip("\"")
 
 MARKET_PROXY=os.environ.setdefault('MARKET_PROXY', 'SPY')
 
+ANALYSIS_MODE=os.environ.setdefault('ANALYSIS_MODE', 'geometric')
+
+ESTIMATION_METHOD=os.environ.setdefault('DEFAULT_ESTIMATION_METHOD', 'moments')
+
 ## SERVICE CONFIGURATION
 ### PRICE_MANAGER CONFIGRUATION
 PRICE_MANAGER = os.environ.setdefault('PRICE_MANAGER', 'alpha_vantage')
@@ -184,11 +203,7 @@ if PRICE_MANAGER == 'alpha_vantage':
     AV_RES_EQUITY_FIRST_LAYER='Time Series (Daily)'
     AV_RES_EQUITY_CLOSE_PRICE="4. close"
     AV_RES_EQUITY_OPEN_PRICE="1. open"
-    AV_RES_EQUITY_KEY="symbol"
     AV_RES_CRYPTO_FIRST_LAYER='Time Series (Digital Currency Daily)'
-    AV_RES_CRYPTO_KEY="currency code"
-    AV_RES_CRYPTO_CLOSE_PRICE=f'4a. close ({static.constants["DENOMINATION"]})'
-    AV_RES_CRYPTO_OPEN_PRICE=f'1a. open ({static.constants["DENOMINATION"]})'
     AV_RES_ERROR='Error Message'
     AV_RES_LIMIT='Note'
     AV_RES_DAY_LIMIT='Information'
